import numpy as np
from risk_manage.Data_Prepare import Data_Prepare
class model_data_parse(Data_Prepare):
  def __init__(self, target, target_u, daypara):
    super().__init__(target, target_u, daypara)
    #self.target = target
    #self.target_u = target_u
    #self.daypara = daypara
    self.df = super(model_data_parse, self).fetch(target) 
    self.diff = super(model_data_parse, self).diff_data(1)
    self.df_np = np.array(self.df["price"]).reshape(-1,1)
    #self.n = np.arange(1,8)


  def parse_fig1(self, labels, n_class):
    """
    Parse Cluster Label Index V2
    """
    index = [[] for i in range(n_class)]
    for k in range(n_class):
      temp0 = list(filter(lambda i: i[1]==k, enumerate(labels)))
      index[k] = [temp[0] for temp in temp0]
    """
    Parse Crash Index V2
    """
    temp = list(filter(lambda i: (abs(i[1])>300), enumerate(self.diff)))
    crash_date_index = [i[0] for i in temp]

    """
    Parse Exact Crash Date
    """
    crash_date_sort = [[] for _ in range(n_class)]
    for outer_i in range(n_class):
      for i in range(len(index[outer_i])):
        if self.df.index[index[outer_i]][i] in self.diff.index[crash_date_index]:
          crash_date_sort[outer_i].append(self.df.index[index[outer_i]][i])
    return index, crash_date_index, crash_date_sort


import numpy as np
import matplotlib.pyplot as plt
from sklearn.mixture import GaussianMixture
import matplotlib as mat
mat.style.use('ggplot')
from risk_manage.model_data_parse import model_data_parse

class cluster_model(model_data_parse):
  def __init__(self, target, target_u, daypara):
    super().__init__(target, target_u, daypara)
    #self.target = target
    #self.daypara = daypara
    #self.df_np = np.array(self.df["price"]).reshape(-1,1)
    self.n = np.arange(1,8)
    self.model = [GaussianMixture(n, covariance_type='full', random_state=0).fit(self.df_np) for n in self.n]
    
  def plot_info_criteria(self):
    plt.plot(self.n, [m.bic(self.df_np) for m in self.model], label='BIC')
    plt.plot(self.n, [m.aic(self.df_np) for m in self.model], label='AIC')
    plt.legend(loc='best')
    plt.xlabel('n_clusters')

  def fit(self, n_clusters):
    #n = 3
    global n_class
    n_class = n_clusters
    gmm = GaussianMixture(n_components=n_class)
    gmm.fit(self.df_np)
    labels = gmm.predict(self.df_np)
    #self.labels = labels
    return labels

  def plot(self):

    index, crash_date_index, crash_date_sort = super(cluster_model, self).parse_fig1(self.fit(n_class),n_class)

    fig1 = plt.figure(figsize=(14,8), constrained_layout=True)
    gs = fig1.add_gridspec(3,3, width_ratios=[1,2,1])

    ax1 = fig1.add_subplot(gs[1,:-1])
    ax2 = fig1.add_subplot(gs[-1,:-1], sharex=ax1) 
    ax3 = fig1.add_subplot(gs[1:,-1])  

    ax1.plot(self.diff.index, self.diff)
    x = range(n_class)
    y = []

    for i in range(n_class):
      ax2.scatter(self.df["Date"][index[i]], self.df["price"][index[i]],cmap="inferno",marker="o", label=i)
      y.append(len(crash_date_sort[i]))

    for i in range(len(crash_date_index)):
      ax1.axvline(self.diff.index[crash_date_index][i],0,ymax=1000,c="lightblue")
      ax2.axvline(self.diff.index[crash_date_index][i],0,ymax=80,c="lightblue")

    ax1.title.set_text("Quote Change by Day of TWII")
    ax1.set_ylabel("Changes")
    ax2.title.set_text("Vix of S&P500")
    ax2.set_ylabel("Vix")

    color_set = ["red","blue","purple","grey","orange"]
    ax2.legend()
    ax3_list = ax3.bar(x,y, width=0.6)
    ax3.title.set_text("Number of Crashes happen in nth Cluster")
    ax3.locator_params(axis='x', integer=True)

    for i in range(n_class):
      ax3_list[i].set_color(color_set[i])


import datetime
from datetime import timedelta
from pandas_datareader import data as data_reader
import pandas as pd

class Data_Prepare:
  def __init__(self, target, target_u, daypara):
    self.target = target
    self.target_u = target_u
    self.daypara = daypara
    
  def parse(func):         
    def wrap(self, target): 
      data = func(self, target)
      dit = {"Date":data.index, "price":data["Close"], "high":data["High"],"low":data["Low"],'vol':data["Volume"]}
      df = pd.DataFrame(dit)
      return df               
    return wrap 
  
  @parse
  def fetch(self, target):
    now = datetime.date.today()
    start = now - datetime.timedelta(days = int(self.daypara))
    data = data_reader.DataReader(target, "yahoo", str(start), str(now))
    return data

  def diff_data(self, window):
    diff = self.fetch(self.target_u)["price"].diff()[window:]
    return diff